unit wfMain;


interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ComCtrls, Menus, E_StgStr, Ole2,
  explbtn, GoToWeb;

type
  TWebMarkForm = class(TForm)
    StatusBar: TStatusBar;
    TreeViewPanel: TPanel;
    TreeView: TTreeView;
    FileOpenDialog: TOpenDialog;
    FileSaveDialog: TSaveDialog;
    Small: TImageList;
    MainMenu: TMainMenu;
    miFile: TMenuItem;
    NewModel1: TMenuItem;
    miFileOpen: TMenuItem;
    miFileSave: TMenuItem;
    miN1: TMenuItem;
    miFileExit: TMenuItem;
    Options1: TMenuItem;
    AddWorkFlowSubClass1: TMenuItem;
    DeleteWorkFlowItem1: TMenuItem;
    mnEdit: TMenuItem;
    Bevel1: TBevel;
    Panel1: TPanel;
    OpenBtn: TExplorerButton;
    ExplorerButton5: TExplorerButton;
    DelBtn: TExplorerButton;
    PropBtn: TExplorerButton;
    BrowseBtn: TExplorerButton;
    SearchBtn: TExplorerButton;
    GoToWeb1: TGoToWeb;
    procedure FileExit(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SaveResource(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure AddWfSubClass(Sender: TObject);
    procedure DeleteWfItem(Sender: TObject);
    procedure TvDeleteNode(Sender: TObject; Node: TTreeNode);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ClearModel(Sender: TObject);
    procedure mnEditClick(Sender: TObject);
    procedure NewModel1Click(Sender: TObject);
    procedure BrowseBtnClick(Sender: TObject);
    procedure TreeViewDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure TreeViewDragDrop(Sender, Source: TObject; X, Y: Integer);
  private
    CurList      : TStringList;
    TreeStorage  : TStructuredStorage;

    Function StoreNode(Var Storage     : IStorage;
                           CNode       : TTreeNode;
                           Ndx         : Integer) : Boolean;

    Function StoreModel(      CNode       : TTreeNode;
                        Const AModel      : String) : Boolean;


    Function AddModelNode(Const S : String) : TTreeNode;

    Function AddChildNode(Const S     : String;
                          Const PNode : TTreeNode;
                          Const SList : TStringList) : TTreeNode;

    Function ReadNode(Const PNode    : TTreeNode;
                      Var   Storage  : IStorage;
                             Ndx     : Integer) : Boolean;

    Function ReadModel(Const AModel : String) : Boolean;

    Procedure SetOptionsMenu(CState : Integer);
    procedure DisplayResources(FileName : String);
    procedure UpdateViewPanel;
  public
    RETURL : string;
  end;

var
  WebMarkForm: TWebMarkForm;

implementation

uses
     WfSub;


{$R *.DFM}
{$R RxIMAGES.RES}

const
  itBitmap: TResType = Controls.rtBitmap; // Reference for duplicate identifier
  CLOSEFOLDER_BMP = 0;
  OPENFOLDER_BMP  = 1;
  GEARS_BMP       = 2;
  WIN95_BMP       = 3;
  ARROW_BMP       = 4;
  PAINT_BMP       = 5;
  INNOVIEW        = 0;
  INTREEVIEW      = 1;
  INLISTVIEW      = 2;
  SNoResSelected = 'No resource selected';

{ Utility Functions }

procedure Error(const ErrMsg: string);
begin
  raise Exception.Create(ErrMsg);
end;

procedure ErrorFmt(const ErrMsg: string; Params: array of const);
begin
  raise Exception.Create(format(ErrMsg, Params));
end;

function Confirm(const AMsg: String): Boolean;
begin
  Result := MessageDlg(AMsg, mtConfirmation, mbYesNoCancel, 0) = idYes;
end;

{ Form Initialization }

procedure TWebMarkForm.FormCreate(Sender: TObject);
begin
  Small.ResourceLoad(itBitmap, 'SmallImages', clOlive);
  OleInitialize(Nil);
  while GoToWeb1.IsAnyBrowser=false do
    if application.messagebox('No Web Browser Found...','Error',mb_iconerror+mb_retrycancel)=idcancel then halt;
end;

procedure TWebMarkForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
//  OleUninitialize;
end;

procedure TWebMarkForm.UpdateViewPanel;
var
  AList : TStringList;
begin
  with TreeView do
  begin
    if Visible and Assigned(Selected) and (Selected.Data <> nil) then
    begin
      AList := TStringList.Create;
      AList.Assign(TStringList(Selected.Data));
      StatusBar.Panels.Items[0].Text := AList[1];
    end;
  end;
end;

procedure TWebMarkForm.ClearModel(Sender: TObject);
begin
  TreeView.Selected := nil;
  TreeView.Items.Clear;
end;

Procedure TWebMarkForm.SetOptionsMenu(CState : Integer);
begin
  Options1.Enabled:=CState<>INNOVIEW;
  Case CState Of
    INTREEVIEW : begin
                   AddWorkFlowSubClass1.Enabled:=True;
                 end;
    INLISTVIEW : begin
                   AddWorkFlowSubClass1.Enabled:=False;
                 end;
  end;
end;

Function TWebMarkForm.AddModelNode(Const S : String) : TTReeNode;
begin
  TreeView.Selected:=Nil;
  Result:=TreeView.Items.Add(Nil,S);
  Result.ImageIndex := 3;//CLOSEFOLDER_BMP;
  Result.SelectedIndex := 3;//OPENFOLDER_BMP;
  TreeView.Selected:=Result;
end;

Function TWebMarkForm.AddChildNode(Const S     : String;
                                   Const PNode : TTreeNode;
                                   Const SList : TStringList) : TTreeNode;
begin
  Result := TreeView.Items.AddChildObJect(PNode,S,SList);
  Result.ImageIndex := CLOSEFOLDER_BMP;
  Result.SelectedIndex := OPENFOLDER_BMP;
end;


Function TWebMarkForm.ReadNode(Const PNode    : TTreeNode;
                            Var   Storage  : IStorage;
                                   Ndx     : Integer) : Boolean;
Var
     SubStorage : IStorage;
     Stream     : IStream;
     I          : Integer;
     S          : String;
     SubNodes   : Integer;
     SubItems   : Integer;
     CNode      : TTreeNode;
begin
  Result:=False;
  With TreeStorage do
    If OpenSubStorage('NODE'+IntToStr(Ndx),STGM_DIRECT OR STGM_READ
                        OR STGM_SHARE_EXCLUSIVE,Storage,SubStorage) then
    begin
      If OpenStream(SubStorage,'DATA',STGM_DIRECT OR STGM_READ
                    OR STGM_SHARE_EXCLUSIVE,Stream) then
      begin
        Result:=ReadString(Stream,S);
        If Result then
          Result:=ReadInt(Stream,SubNodes);
        If Result then
        begin
          Result:=ReadInt(Stream,SubItems);
          If Result then
          begin
            If SubItems>0 then
              CurList:=TStringList.Create
            else
              CurList:=Nil;
            CNode:=AddChildNode(S,PNode,CurList);
            If SubItems>0 then
            begin
              For I:=0 to SubItems-1 do
              begin
                Result:=ReadString(Stream,S);
                If Result then
                  CurList.Add(S)
                else
                  Break;
              end;
              CurList:=Nil;
            end;
          end;
        end;
        DoneStream(Stream);
      end;
      If Result then
        For I:= 0 to SubNodes-1 do
        begin
          Result:=ReadNode(CNode,SubStorage,I);
          If NOT Result then
            Break;
        end;
      DoneStorage(SubStorage);
    end;
end;

Function TWebMarkForm.ReadModel(Const AModel : String) : Boolean;
Var
     Storage  : IStorage;
     Stream   : IStream;
     I        : Integer;
     SubNodes : Integer;
     S        : String;
     CNode    : TTreeNode;
begin
  Result:=False;
  With TreeStorage do
    If OpenSubStorage(AModel,STGM_DIRECT OR STGM_READ
                      OR STGM_SHARE_EXCLUSIVE,StorageFile,Storage) then
    begin
      SubNodes:=0;
      If OpenStream(Storage,'DATA',STGM_DIRECT OR STGM_READ
                    OR STGM_SHARE_EXCLUSIVE,Stream) then
      begin
        Result:=ReadString(Stream,S);
        If Result then
          Result:=ReadInt(Stream,SubNodes);
        DoneStream(Stream);
      end;
      If Result then
      begin
        CNode:=AddModelNode(S);
        For I:= 0 to SubNodes-1 do
        begin
          Result:=ReadNode(CNode,Storage,I);
          If NOT Result then
            Break;
        end;
      end;
      DoneStorage(Storage);
    end;
end;

procedure TWebMarkForm.DisplayResources(FileName : String);
Var
    Stream      : IStream;
    I           : Integer;
    Success     : Boolean;
    NumModels   : Integer;
    S           : String;
begin
  TreeStorage:=TStructuredStorage.Create;
  Try
    With TreeStorage do
      If IsStorageFile(FileName) then
      begin
        ClearModel(Self);
        If OpenStorageFile(FileName,STGM_DIRECT OR STGM_READ OR
                                    STGM_SHARE_DENY_WRITE) then
        begin
          If OpenStream(StorageFile,'MODELS',STGM_DIRECT OR STGM_READ
                                    OR STGM_SHARE_EXCLUSIVE,Stream) then
          begin
            Success:=ReadInt(Stream,NumModels);
            DoneStream(Stream);
            If Success then
            begin
              For I:=0 to NumModels-1 do
                ReadModel('MODEL'+IntToStr(I));
            end
            else; {Error}
          end
          else ; {Do An Error}
        end
        else ; {Error}
        with TreeView do
        begin
          // SetFocus;
          Selected := Items[0];
          Selected.Expanded := True;
        end;
      end
      else ; {Do an Error}
  Except
    TreeStorage.Free;
    Raise;
  end;
  TreeStorage.Free;

end;

procedure TWebMarkForm.FileOpen(Sender: TObject);
var
  FileName : string;
begin
  FileName := ExtractFilePath(Application.ExeName)+'index.wmk';
  if FileExists(FileName) then DisplayResources(FileName)
  else with FileOpenDialog do
  begin
    if not Execute then Exit;
    DisplayResources(FileName);
  end;
end;


Function TWebMarkForm.StoreNode(Var   Storage     : IStorage;
                                   CNode       : TTreeNode;
                                   Ndx         : Integer) : Boolean;
Var
     SubStorage : IStorage;
     Stream     : IStream;
     I          : Integer;
begin
  Result:=False;
  With TreeStorage do
    If CreateSubStorage('NODE'+IntToStr(Ndx),STGM_DIRECT OR STGM_CREATE OR STGM_WRITE
                        OR STGM_SHARE_EXCLUSIVE,Storage,SubStorage) then
    begin
      If CreateStream(SubStorage,'DATA',STGM_DIRECT OR STGM_CREATE
                      OR STGM_WRITE OR STGM_SHARE_EXCLUSIVE,Stream) then
      begin
        Result:=WriteString(Stream,CNode.Text);
        If Result then
          Result:=WriteInt(Stream,CNode.Count);
        If Result then
          If CNode.Data<>Nil then
          begin
            CurList:=TStringList(CNode.Data);
            Result:=WriteInt(Stream,CurList.Count);
            If Result then
              For I:=0 to CurList.Count-1 do
                WriteString(Stream,CurList.Strings[I]);
            CurList:=Nil;
          end
          else
            Result:=WriteInt(Stream,0);
        DoneStream(Stream);
      end;
      With CNode do
       For I:= 0 to Count-1 do
       begin
         Result:=StoreNode(SubStorage,Item[I],I);
         If NOT Result then
           Break;
       end;
      DoneStorage(SubStorage);
    end;
end;

Function TWebMarkForm.StoreModel(      CNode  : TTreeNode;
                              Const AModel : String) : Boolean;
Var
     Storage : IStorage;
     Stream  : IStream;
     I       : Integer;
begin
  Result:=False;
  With TreeStorage do
    If CreateSubStorage(AModel,STGM_DIRECT OR STGM_CREATE OR STGM_WRITE
                        OR STGM_SHARE_EXCLUSIVE,StorageFile,Storage) then
    begin
      If CreateStream(Storage,'DATA',STGM_DIRECT OR STGM_CREATE
                      OR STGM_WRITE OR STGM_SHARE_EXCLUSIVE,Stream) then
      begin
        Result:=WriteString(Stream,CNode.Text);
        If Result then
          Result:=WriteInt(Stream,CNode.Count);
        DoneStream(Stream);
      end;
      With CNode do
       For I:= 0 to Count-1 do
       begin
         Result:=StoreNode(Storage,Item[I],I);
         If NOT Result then
           Break;
       end;
      DoneStorage(Storage);
    end;
end;

procedure TWebMarkForm.SaveResource(Sender: TObject);
Var
    Stream      : IStream;
    I           : Integer;
    Success     : Boolean;
    ModelCount  : Integer;
begin
  If TreeView.Items.Count>0 then
  begin
{    If ActiveControl = ListView then
      UpdateTreeView;}
    Success:=False;
    FileSaveDialog.FileName:=FileOpenDialog.FileName;
    with FileSaveDialog do
      if Execute then
      begin
        TreeStorage:=TStructuredStorage.Create;
        Try
          If TreeStorage.CreateStorageFile(FileName,STGM_DIRECT OR STGM_CREATE OR STGM_WRITE
                                           OR STGM_SHARE_EXCLUSIVE) then
          begin
            CurList:=Nil;
            With TreeStorage do
            begin
              If CreateStream(StorageFile,'MODELS',STGM_DIRECT OR STGM_CREATE
                              OR STGM_WRITE OR STGM_SHARE_EXCLUSIVE,Stream) then
              begin
                ModelCount:=0;
                For I:=0 to TreeView.Items.Count-1 do
                  If TreeView.Items[I].Parent=Nil then
                    Inc(ModelCount);
                Success:=WriteInt(Stream,ModelCount);
                If Success then
                  For I:=0 to TreeView.Items.Count-1 do
                  begin
                    If TreeView.Items[I].Parent=Nil then
                    begin
                      Success:=WriteString(Stream,TreeView.Items[I].Text);
                      IF NOT Success then
                        Break;
                    end;
                  end;
                DoneStream(Stream);
              end
              else ; {Do An Error}
              ModelCount:=0;
              If Success then
                For I:=0 to TreeView.Items.Count-1 do
                  If TreeView.Items[I].Parent=Nil then
                    If NOT StoreModel(TreeView.Items[I],'MODEL'+IntToStr(ModelCount)) then
                    begin
                      Success:=False;
                      Break;
                    end
                    else
                      Inc(ModelCount);
            end;
          end
          else{Do An Error}
          begin
          end;
        except
          TreeStorage.Free;
          Raise;
        end;
        TreeStorage.Free;
      end;
    If Success then
      FileOpenDialog.FileName:=FileSaveDialog.FileName;
  end;
end;

procedure TWebMarkForm.FileExit(Sender: TObject);
begin
  Close;
end;

procedure TWebMarkForm.TreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  UpdateViewPanel;
  if TreeView.Selected <> nil then
    if TreeView.Selected.Parent = nil then begin
      DelBtn.Enabled := False;
      PropBtn.Enabled := False;
    end
    else begin
      DelBtn.Enabled := True;
      PropBtn.Enabled := True;
    end;
end;

procedure TWebMarkForm.AddWfSubClass(Sender: TObject);
Var
    ClsName : String;
    AStrings  : TStringList;
    i : integer;
begin
  If (ActiveControl=TreeView) AND (TreeView.Selected<>Nil) then
    If WfSubClassForm.ShowModal=MrOk then
    begin
      AStrings:=Nil;
      With WfSubClassForm do
      begin
        ClsName:=EditClassName.Text;
        AStrings:=TStringList.Create;
        AStrings.Add('0');
        AStrings.Add(URLEdit.Text);
        for i := 0 to Memo1.Lines.Count-1 do AStrings.Add(Memo1.Lines[i]);
      end;
      If ClsName='' then
        ClsName:='NEW WEBMARK ITEM';
      TreeView.Items.AddChildObJect(TreeView.Selected,ClsName,AStrings);
    end;
end;

procedure TWebMarkForm.DeleteWfItem(Sender: TObject);
begin
  If ACtiveControl=TreeView then
    If (TreeView.Selected<>Nil) and
       (TreeView.Selected.Parent<>nil) then
    begin
      TreeView.Selected.DeleteChildren;
      TreeView.Selected.Delete;
    end;
end;

procedure TWebMarkForm.TvDeleteNode(Sender: TObject; Node: TTreeNode);
begin
  If Node.Data<>Nil then
    TStringList(Node.Data).Free;
end;

procedure TWebMarkForm.mnEditClick(Sender: TObject);
Var
  ClsName : String;
  AStrings : TStringList;
  i : integer;
begin
  If (ActiveControl=TreeView) AND (TreeView.Selected<>Nil)
     and (TreeView.Selected.Parent <> nil) then begin
    AStrings:=TStringList.Create;
    with wfSubClassForm do begin
      EditClassName.Text := TreeView.Selected.Text;
      AStrings.Assign(TStringList(TreeView.Selected.Data));
      //TypeRadio.ItemIndex := StrToInt(AStrings[0]);
      URLEdit.Text := AStrings[1];
      Memo1.Clear;
      for i := 2 to AStrings.Count-1 do Memo1.Lines.Add(AStrings[i]);
    end;
    If WfSubClassForm.ShowModal=MrOk then
    begin
      AStrings.Clear;
      With WfSubClassForm do
      begin
        ClsName:=EditClassName.Text;
        AStrings.Add('0');
        AStrings.Add(URLEdit.Text);
        for i := 0 to Memo1.Lines.Count-1 do AStrings.Add(Memo1.Lines[i]);
      end;
      If ClsName='' then
        ClsName:='NEW WEBMARK ITEM';
      TreeView.Selected.Text := ClsName;
      TreeView.Selected.Data := AStrings;
    end;
    UpdateViewPanel;
  end;
end;

procedure TWebMarkForm.NewModel1Click(Sender: TObject);
begin
  ClearModel(Sender);
  AddModelNode('My Web Mark Link');
end;

procedure TWebMarkForm.BrowseBtnClick(Sender: TObject);
var
  AList : TStringList;
begin
  with TreeView do
  begin
    if Visible and Assigned(Selected) and (Selected.Data <> nil) then
    begin
      AList := TStringList.Create;
      AList.Assign(TStringList(Selected.Data));
      RETURL := AList[1];
      GotoWeb1.URL := RETURL;
      if GoToWeb1.Execute=false then
        application.messagebox('No Browser is accessible.','Error',mb_iconerror+mb_ok);
    end;
  end;
  ModalResult := mrOK;
end;

procedure TWebMarkForm.TreeViewDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := (Source is TTreeView) and (TreeView.GetNodeAt(X, Y) <> nil)
    and (TreeView.Selected <> TreeView.GetNodeAt(X, Y));
end;

procedure TWebMarkForm.TreeViewDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  AnItem: TTreeNode;
  AttachMode: TNodeAttachMode;
  HT: THitTests;
begin
  if TreeView.Selected = nil then Exit;
  HT := TreeView.GetHitTestInfoAt(X, Y);
  AnItem := TreeView.GetNodeAt(X, Y);
  if (HT - [htOnItem, htOnIcon, htNowhere, htOnIndent] <> HT) and
     (AnItem <> TreeView.Selected) then
  begin
    AttachMode := naAdd;
    if (htOnItem in HT) or (htOnIcon in HT) then AttachMode := naAddChild
    else if htNowhere in HT then AttachMode := naAdd

    else if htOnIndent in HT then AttachMode := naInsert;
    TreeView.Selected.MoveTo(AnItem, AttachMode);
  end;

end;

end.
